if Code.ensure_loaded?(NimbleCSV) do
  defmodule Clickhouse.Format.TSVWithNames do
    @moduledoc """
    An implementation of the Clickhouse `"TSVWithNames"`/`"TabSeparatedValuesWithNames"` format.
    """

    @behaviour Clickhouse.Format

    alias Clickhouse.{Format.TSV, Result}

    @parser TSV.Parser

    ################################
    # Clickhouse.Format Callbacks
    ################################

    @impl Clickhouse.Format
    @spec decode(Clickhouse.Result.t()) :: {:ok, Clickhouse.Result.t()}
    def decode(result) do
      {columns, data} = do_decode(result)
      result = Result.finalize(result, columns, data)
      {:ok, result}
    end

    @impl Clickhouse.Format
    @spec encode(Clickhouse.Query.t()) ::
            {:ok, Clickhouse.Query.t()} | {:error, Clickhouse.Error.t()}
    def encode(query), do: TSV.encode(query)

    ################################
    # Private API
    ################################

    defp do_decode(result) do
      [names | data] = @parser.parse_string(result.raw, skip_headers: false)
      columns = Enum.map(names, fn name -> {name, nil} end)
      {columns, data}
    end
  end
end
