defmodule Clickhouse.Result do
  @moduledoc """
  The results of a Clickhouse query.
  """

  alias Clickhouse.Config

  @type name :: binary()
  @type type :: atom() | nil
  @type columns :: [{name(), type()}, ...] | nil
  @type data :: [] | [list(), ...]
  @type t :: %__MODULE__{
          client: Clickhouse.client(),
          raw: binary(),
          raw_format: binary(),
          format: Clickhouse.format(),
          columns: columns(),
          data: data()
        }

  @derive {Inspect, only: [:data, :columns]}
  defstruct [:client, :raw, :raw_format, :format, :columns, data: []]

  @spec new(Clickhouse.client(), binary(), binary()) :: Clickhouse.Result.t()
  def new(client, raw, raw_format) do
    %__MODULE__{client: client, raw: raw, raw_format: raw_format}
    |> get_format()
  end

  @spec finalize(Clickhouse.Result.t(), columns(), data()) :: Clickhouse.Result.t()
  def finalize(result, columns, data) do
    %{result | columns: columns, data: data}
  end

  defp get_format(result) do
    case Config.format(result.client, result.raw_format) do
      {:ok, format} -> %{result | format: format}
      _ -> result
    end
  end
end
